@@ -49,6 +49,26 @@ M_chm_factor_to_SEXP(CHM_FR f, int dofree)
     return fun(f, dofree);
 }
 
+double attribute_hidden
+M_chm_factor_ldetL2(CHM_FR f)
+{
+    static double(*fun)(CHM_FR) = NULL;
+    if(fun == NULL)
+	fun = (double(*)(CHM_FR))
+	    R_GetCCallable("Matrix", "chm_factor_ldetL2");
+    return fun(f);
+}
+
+CHM_FR attribute_hidden
+M_chm_factor_update(CHM_FR f, CHM_SP A, double mult)
+{
+    static CHM_FR(*fun)(CHM_FR,CHM_SP,double) = NULL;
+    if(fun == NULL)
+	fun = (CHM_FR(*)(CHM_FR,CHM_SP,double))
+	    R_GetCCallable("Matrix", "chm_factor_update");
+    return fun(f, A, mult);
+}
+
 SEXP attribute_hidden
 M_chm_sparse_to_SEXP(CHM_SP a, int dofree,
 		     int uploT, int Rkind, char *diag, SEXP dn)
